import { RxBaseTemplateNames, createTemplateManager } from '@rx-angular/cdk/template';
import * as i0 from '@angular/core';
import { Directive, Input, NgModule } from '@angular/core';
import { coerceAllFactory, coerceObservable } from '@rx-angular/cdk/coercing';
import { createTemplateNotifier } from '@rx-angular/cdk/notifications';
import * as i1 from '@rx-angular/cdk/render-strategies';
import { Subscription, ReplaySubject, Subject } from 'rxjs';
import { mergeAll } from 'rxjs/operators';

const RxIfTemplateNames = Object.assign(Object.assign({}, RxBaseTemplateNames), { then: 'rxThen', else: 'rxElse' });

// eslint-disable-next-line @angular-eslint/directive-class-suffix
class RxIf {
    constructor(strategyProvider, cdRef, ngZone, thenTemplateRef, viewContainerRef) {
        this.strategyProvider = strategyProvider;
        this.cdRef = cdRef;
        this.ngZone = ngZone;
        this.thenTemplateRef = thenTemplateRef;
        this.viewContainerRef = viewContainerRef;
        this.subscription = new Subscription();
        this.renderParent = this.strategyProvider.config.parent;
        this.patchZone = this.strategyProvider.config.patchZone;
        /** @internal */
        this.observablesHandler = createTemplateNotifier();
        this.strategyHandler = coerceAllFactory(() => new ReplaySubject(1), mergeAll());
        this.rendered$ = new Subject();
    }
    set rxIf(potentialObservable) {
        this.observablesHandler.next(coerceObservable(potentialObservable));
    }
    set strategy(strategyName) {
        this.strategyHandler.next(strategyName);
    }
    set renderCallback(callback) {
        this._renderObserver = callback;
    }
    ngOnInit() {
        this.subscription.add(this.templateManager
            .render(this.observablesHandler.values$)
            .subscribe((n) => {
            var _a;
            this.rendered$.next(n);
            (_a = this._renderObserver) === null || _a === void 0 ? void 0 : _a.next(n);
        }));
    }
    /** @internal */
    ngOnChanges(changes) {
        if (!this.templateManager) {
            this._createTemplateManager();
        }
        if (changes.else) {
            this.templateManager.addTemplateRef(RxIfTemplateNames.else, this.else);
        }
        if (changes.complete) {
            this.templateManager.addTemplateRef(RxIfTemplateNames.complete, this.complete);
        }
        if (changes.suspense) {
            this.templateManager.addTemplateRef(RxIfTemplateNames.suspense, this.suspense);
        }
        if (changes.error) {
            this.templateManager.addTemplateRef(RxIfTemplateNames.error, this.error);
        }
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
    }
    _createTemplateManager() {
        this.templateManager = createTemplateManager({
            templateSettings: {
                viewContainerRef: this.viewContainerRef,
                customContext: (rxIf) => ({ rxIf }),
            },
            renderSettings: {
                cdRef: this.cdRef,
                parent: coerceBooleanProperty(this.renderParent),
                patchZone: this.patchZone ? this.ngZone : false,
                defaultStrategyName: this.strategyProvider.primaryStrategy,
                strategies: this.strategyProvider.strategies,
            },
            notificationToTemplateName: {
                ["suspense" /* Suspense */]: () => RxIfTemplateNames.suspense,
                ["next" /* Next */]: (value, templates) => {
                    return value
                        ? RxIfTemplateNames.then
                        : templates.get(RxIfTemplateNames.else)
                            ? RxIfTemplateNames.else
                            : undefined;
                },
                ["error" /* Error */]: () => RxIfTemplateNames.error,
                ["complete" /* Complete */]: () => RxIfTemplateNames.complete,
            },
        });
        this.templateManager.addTemplateRef(RxIfTemplateNames.then, this.thenTemplateRef);
        this.templateManager.nextStrategy(this.strategyHandler.values$);
    }
}
/** @nocollapse */ RxIf.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: RxIf, deps: [{ token: i1.RxStrategyProvider }, { token: i0.ChangeDetectorRef }, { token: i0.NgZone }, { token: i0.TemplateRef }, { token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ RxIf.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.16", type: RxIf, selector: "[rxIf]", inputs: { rxIf: "rxIf", strategy: ["rxIfStrategy", "strategy"], else: ["rxIfElse", "else"], suspense: ["rxIfSuspense", "suspense"], complete: ["rxIfComplete", "complete"], error: ["rxIfError", "error"], renderParent: ["rxIfParent", "renderParent"], patchZone: ["rxIfPatchZone", "patchZone"], renderCallback: ["rxIfRenderCallback", "renderCallback"] }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: RxIf, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rxIf]',
                }]
        }], ctorParameters: function () { return [{ type: i1.RxStrategyProvider }, { type: i0.ChangeDetectorRef }, { type: i0.NgZone }, { type: i0.TemplateRef }, { type: i0.ViewContainerRef }]; }, propDecorators: { rxIf: [{
                type: Input
            }], strategy: [{
                type: Input,
                args: ['rxIfStrategy']
            }], else: [{
                type: Input,
                args: ['rxIfElse']
            }], suspense: [{
                type: Input,
                args: ['rxIfSuspense']
            }], complete: [{
                type: Input,
                args: ['rxIfComplete']
            }], error: [{
                type: Input,
                args: ['rxIfError']
            }], renderParent: [{
                type: Input,
                args: ['rxIfParent']
            }], patchZone: [{
                type: Input,
                args: ['rxIfPatchZone']
            }], renderCallback: [{
                type: Input,
                args: ['rxIfRenderCallback']
            }] } });
function createViewContext(value) {
    return {
        rxIf: value,
        $implicit: value,
        error: false,
        complete: false,
        suspense: false,
    };
}
function updateViewContext(value, view, context) {
    Object.keys(context).forEach((k) => {
        view.context[k] = context[k];
    });
}
/**
 * Coerces a data-bound value (typically a string) to a boolean.
 */
function coerceBooleanProperty(value) {
    return value != null && `${value}` !== 'false';
}

const DECLARATIONS = [RxIf];
class IfModule {
}
/** @nocollapse */ IfModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IfModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ IfModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IfModule, declarations: [RxIf], exports: [RxIf] });
/** @nocollapse */ IfModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IfModule, imports: [[]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IfModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: DECLARATIONS,
                    imports: [],
                    exports: DECLARATIONS,
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { IfModule, RxIf, RxIfTemplateNames };
//# sourceMappingURL=template-experimental-if.js.map
